@@ -557,7 +557,7 @@ static inline void fio_ro_check(struct thread_data *td, struct io_u *io_u)
 #define RAND_MAP_IDX(f, b)	(TO_MAP_BLOCK(f, b) / BLOCKS_PER_MAP)
 #define RAND_MAP_BIT(f, b)	(TO_MAP_BLOCK(f, b) & (BLOCKS_PER_MAP - 1))
 
-#define MAX_JOBS	(1024)
+#define MAX_JOBS	(2048)
 
 #define td_non_fatal_error(e)	((e) == EIO || (e) == EILSEQ)
 
@@ -666,7 +666,7 @@ extern int load_blktrace(struct thread_data *, const char *);
 	if (!(cond)) {			\
 		int *__foo = NULL;	\
 		fprintf(stderr, "file:%s:%d, assert %s failed\n", __FILE__, __LINE__, #cond);	\
-		(td)->runstate = TD_EXITED;	\
+		td_set_runstate((td), TD_EXITED);	\
 		(td)->error = EFAULT;		\
 		*__foo = 0;			\
 	}	\
