@@ -10,9 +10,10 @@
 #undef rb_left
 #undef rb_right
 
+#include "../file.h"
+
 #define FIO_HAVE_POSIXAIO
 #define FIO_HAVE_FADVISE
-#define	fadvise	posix_fadvise	/* XXX NetBSD doesn't have fadvise */
 #define FIO_HAVE_ODIRECT
 #define FIO_HAVE_STRSEP
 #define FIO_HAVE_FDATASYNC
@@ -23,9 +24,13 @@
 
 #define OS_MAP_ANON		MAP_ANON
 
+#ifndef PTHREAD_STACK_MIN
+#define PTHREAD_STACK_MIN 4096
+#endif
+
 typedef off_t off64_t;
 
-static inline int blockdev_invalidate_cache(int fd)
+static inline int blockdev_invalidate_cache(struct fio_file *f)
 {
 	return EINVAL;
 }
